@@ -62,8 +62,8 @@ DDocPrivate::DDocPrivate()
 ,	f_createSignedDoc(0)
 ,	f_DataFile_delete(0)
 ,	f_DataFile_new(0)
-,	f_ddocReadNewSignaturesFromDdoc(0)
 ,	f_ddocSaxReadSignedDocFromFile(0)
+,	f_ddocSigInfo_GetOCSPRespondersCert(0)
 ,	f_ddocSigInfo_GetSignersCert(0)
 ,	f_finalizeDigiDocLib(0)
 ,	f_getErrorString(0)
@@ -136,8 +136,8 @@ bool DDocPrivate::loadSymbols()
 	(f_createSignedDoc = (sym_createSignedDoc)lib.resolve("createSignedDoc")) &&
 	(f_DataFile_delete = (sym_DataFile_delete)lib.resolve("DataFile_delete")) &&
 	(f_DataFile_new = (sym_DataFile_new)lib.resolve("DataFile_new")) &&
-	(f_ddocReadNewSignaturesFromDdoc = (sym_ddocReadNewSignaturesFromDdoc)lib.resolve("ddocReadNewSignaturesFromDdoc")) &&
 	(f_ddocSaxReadSignedDocFromFile = (sym_ddocSaxReadSignedDocFromFile)lib.resolve("ddocSaxReadSignedDocFromFile")) &&
+	(f_ddocSigInfo_GetOCSPRespondersCert = (sym_ddocSigInfo_GetOCSPRespondersCert)lib.resolve("ddocSigInfo_GetOCSPRespondersCert")) &&
 	(f_ddocSigInfo_GetSignersCert = (sym_ddocSigInfo_GetSignersCert)lib.resolve("ddocSigInfo_GetSignersCert")) &&
 	(f_finalizeDigiDocLib = (sym_finalizeDigiDocLib)lib.resolve("finalizeDigiDocLib")) &&
 	(f_getErrorString = (sym_getErrorString)lib.resolve("getErrorString")) &&
@@ -263,6 +263,25 @@ std::string SignatureDDOC::getMediaType() const
 	return s.str();
 }
 
+X509Cert SignatureDDOC::getOCSPCertificate() const
+{
+	try { return X509Cert( m_doc->f_ddocSigInfo_GetOCSPRespondersCert( m_doc->doc->pSignatures[m_id] ) ); }
+	catch( const Exception & ) {}
+	return X509Cert();
+}
+
+std::string SignatureDDOC::getProducedAt() const
+{
+	NotaryInfo *n = m_doc->doc->pSignatures[m_id]->pNotary;
+	return n ? std::string( n->timeProduced ) : std::string();
+}
+
+std::string SignatureDDOC::getResponderID() const
+{
+	NotaryInfo *n = m_doc->doc->pSignatures[m_id]->pNotary;
+	return n ? std::string( (const char*)n->mbufRespId.pMem, n->mbufRespId.nLen ) : std::string();
+}
+
 void SignatureDDOC::getRevocationOCSPRef(std::vector<unsigned char>& data, std::string& digestMethodUri) const throw(SignatureException)
 {
 	NotaryInfo *n = m_doc->doc->pSignatures[m_id]->pNotary;
@@ -473,10 +492,37 @@ void DDoc::sign( Signer *signer, Signature::Type type ) throw(BDocException)
 	if ( type == Signature::MOBILE )
 	{
 		std::string file = signer->signaturePath();
-		d->f_ddocReadNewSignaturesFromDdoc(d->doc, file.c_str());
+		SignedDoc *sigDoc = 0;
+		int err = d->f_ddocSaxReadSignedDocFromFile( &sigDoc, file.c_str(), 0, 0 );
+		d->throwError( err, "Failed to sign document", __LINE__ );
+
+		SignatureInfo **signatures = (SignatureInfo**)realloc( d->doc->pSignatures,
+			(d->doc->nSignatures + sigDoc->nSignatures) * sizeof(void *));
+		if( !signatures )
+		{
+			d->f_SignedDoc_free( sigDoc );
+			d->throwError( "Failed to sign document", __LINE__ );
+		}
+
+		d->doc->pSignatures = signatures;
+		for( int i = 0; i < sigDoc->nSignatures; ++i )
+		{
+			d->doc->pSignatures[d->doc->nSignatures + i] = sigDoc->pSignatures[i]; // take ownership
+			sigDoc->pSignatures[i] = 0;
+			// from ddocReadNewSignaturesFromDdoc
+			((char*)d->doc->pSignatures[d->doc->nSignatures + i]->pDocs[0]->szDigest)[0] = 0x0A;
+		}
+		d->doc->nSignatures += sigDoc->nSignatures;
+		sigDoc->nSignatures = 0;
+
+		d->f_SignedDoc_free( sigDoc );
 		d->loadSignatures();
 		return;
 	}
+	PKCS11SignerAbstract *pkcs11 = 0;
+	if( signer->type() != PKCS11SignerAbstract::Type ||
+		!(pkcs11 = static_cast<PKCS11SignerAbstract*>(signer)) )
+		d->throwError( "Failed to sign document", __LINE__ );
 
 	SignatureInfo *info;
 	int err = d->f_SignatureInfo_new( &info, d->doc, NULL );
@@ -484,13 +530,13 @@ void DDoc::sign( Signer *signer, Signature::Type type ) throw(BDocException)
 	err = d->f_addAllDocInfos( d->doc, info );
 	d->throwSignError( info->szId, err, "Failed to sign document", __LINE__ );
 
-	Signer::SignatureProductionPlace l = signer->getSignatureProductionPlace();
+	Signer::SignatureProductionPlace l = pkcs11->getSignatureProductionPlace();
 	err = d->f_setSignatureProductionPlace( info, l.city.c_str(), l.stateOrProvince.c_str(),
 		l.postalCode.c_str(), l.countryName.c_str() );
 	d->throwSignError( info->szId, err, "Failed to sign document", __LINE__ );
 
 	std::ostringstream role;
-	Signer::SignerRole::TRoles r = signer->getSignerRole().claimedRoles;
+	Signer::SignerRole::TRoles r = pkcs11->getSignerRole().claimedRoles;
 	for( Signer::SignerRole::TRoles::const_iterator i = r.begin(); i != r.end(); ++i )
 	{
 		role << *i;
@@ -502,30 +548,24 @@ void DDoc::sign( Signer *signer, Signature::Type type ) throw(BDocException)
 
 	std::string pin;
 	int slot = d->f_ConfigItem_lookup_int( "DIGIDOC_SIGNATURE_SLOT", 0 );
-	if( PKCS11Signer *pkcs11 = static_cast<PKCS11Signer*>(signer) )
+	try
 	{
-		try
-		{
-			PKCS11Signer::PKCS11Cert c;
-			c.cert = pkcs11->getCert();
-			slot = pkcs11->slotNumber();
-			pin = pkcs11->getPin( c );
-			pkcs11->unloadDriver();
-		}
-		catch( const Exception &e )
-		{
-			d->f_SignatureInfo_delete( d->doc, info->szId );
-			throw BDocException( __FILE__, __LINE__, "Failed to sign document", e );
-		}
+		PKCS11SignerAbstract::PKCS11Cert c;
+		c.cert = pkcs11->getCert();
+		slot = pkcs11->slotNumber();
+		pin = pkcs11->getPin( c );
+		pkcs11->unloadDriver();
+	}
+	catch( const Exception &e )
+	{
+		d->f_SignatureInfo_delete( d->doc, info->szId );
+		throw BDocException( __FILE__, __LINE__, "Failed to sign document", e );
 	}
 	err = d->f_calculateSignatureWithEstID( d->doc, info, slot, pin.c_str() );
 	fill(pin.begin(),pin.end(),0);
 
-	if( PKCS11Signer *pkcs11 = static_cast<PKCS11Signer*>(signer) )
-	{
-		try { pkcs11->loadDriver(); }
-		catch( const Exception & ) {}
-	}
+	try { pkcs11->loadDriver(); }
+	catch( const Exception & ) {}
 
 	d->throwSignError( info->szId, err, "Failed to sign document", __LINE__ );
 
